/*************************************************************
 * キャラクタ液晶表示器　(SC1602）
 * シリアル通信から取得したデータを液晶に表示するプログラム
 * 20x4行用
 * 使用PIC16F1938
 * 使用コンパイラ　ＸＣ８
 * 2023年4月23日 
 *************************************************************/
#include    <xc.h>
#include    <stdio.h>
#include    "lcd_lib.h"

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON      // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)

/** 内部クロック　８ＭＨｚ **/
#define  _XTAL_FREQ     8000000	//DELAY 用

/********　変数定義　************/
#define	LINE1			0x80    // １行目設定
#define	LINE2			0xc0    // ２行目設定
#define	LINE3			0x94    // ３行目設定
#define	LINE4			0xd4    // ４行目設定

#define UARTPRINT       0       //シリアル出力に設定
#define LCDPRINT        1       //ＬＣＤ表示に設定

unsigned	char Buffer[42];	//
unsigned	int t;              //受信文字数					
unsigned	int series_flg;     //文字列完了フラグ

unsigned    int     p_mode;      //出力先切り替え

/******** サブ関数   **************/
void putch(unsigned char byte);

/******** メイン開始  ************/
void main(void) {
    /** 初期設定 **/
    OSCCON = 0x72;              //　内臓クロック８HMz指定(BPLL Off)
    /**ポートの設定 ***/
    ANSELA = 0x00;              //A/D停止
    ANSELB = 0x00;              //A/D停止
    TRISA = 0x3B;               //ポート設定 
    TRISB = 0x00;               // 
    TRISC = 0x80;               // 
  
    /**　シリアル通信の設定 **/
    TXSTA = 0x26;				//シリアル受信設定
    RCSTA = 0x90;				//シリアル送信設定
	BAUDCON=0x08;				//
	SPBRG = 207;				//ボーレート  9600bps
    
    PEIE = 1;					//基本モジュール　タイマー０・外部割り込み以外の割り込み許可
	RCIE = 1;					//受信割り込み許可
	GIE = 1;					//すべての割り込み許可

	RCIF = 0;                  // 割り込みフラグをクリア

 /************　ＬＣＤモジュール 表示テストプログラム　********************/
    lcd_init();                 //　LCD初期化
    lcd_cmd(0x01);              //　表示クリア
    lcd_cmd(0x0c);              //　ブリンク非表示
 
    p_mode = UARTPRINT;         //　出力先をシリアル通信にする
    printf("UART -> LCD Ver.1a \r\n");//  テキスト表示

    p_mode = LCDPRINT;              //　出力先をLCD表示にする    
    lcd_cmd(LINE1);                 //　カーソルを１行目の先頭にする。
    printf("  2023/04/23");         //  テキスト表示
    lcd_cmd(LINE2);                 //  カーソルを２行目の先頭にする。
    printf("    UART -> LCD ");     //  テキスト表示
		__delay_ms(9000);           //  250ms ウエイト
        
    while (1) 
	{
		if(series_flg == 1)         //文字列受信完了か？
		{
			series_flg = 0;         //フラグをクリア
            p_mode = UARTPRINT;     //　出力先をシリアル通信にする
		
        printf("\r\n rev = %s \r\n",Buffer);	//文字列を表示
            
            p_mode = LCDPRINT;      //　出力先をLCD表示にする
			if (Buffer[0] == 'L')
			{
				switch(Buffer[1])	
				{
					case '1':
						lcd_cmd(LINE1);				//表示する行をクリアする
						printf("                    ");
						lcd_cmd(LINE1);				//1行目の先頭へ
						printf("%s",Buffer+3);
						break;
						
					case '2':
						lcd_cmd(LINE2);				//表示する行をクリアする
						printf("                    ");
						lcd_cmd(LINE2);				//２行目の先頭へ
						printf("%s",Buffer+3);
						break;

					case '3':
						lcd_cmd(LINE3);				//表示する行をクリアする
						printf("                    ");
						lcd_cmd(LINE3);				//３行目の先頭へ
						printf("%s",Buffer+3);
						break;

					case '4':
						lcd_cmd(LINE4);				//表示する行をクリアする
						printf("                    ");
						lcd_cmd(LINE4);				//４行目の先頭へ
						printf("%s",Buffer+3);
						break;
                        
					default:
                        p_mode = UARTPRINT;         //　出力先をシリアル通信にする
						printf("ERR!\r\n");			//デバッグ用	
						break;
				}  //switch end
			}	 // 'L' 時の処理ｅｎｄ				          
		}
    }
}

/*************************************
 *  1文字出力
 *************************************/

void putch(unsigned char data){

    //UART　出力
    if (p_mode == 0 )
    {
		RCSTAbits.SPEN=1;
		TXREG=data;
		while(!TXIF)continue;
		TXIF=0;
    }   
    //LCD　出力
    else
    {
        lcd_data(data);         //       
    }
}

/*************************************************************
 * 受信割り込み関数
 *１バイトでも受信が有ると割り込みが発生。
 * Buffer[]へ受信データを移す。
 *改行もしくは復帰があると、（リターンキーの事）
 * その時点でBufferを今まで入力された文字を文字列として扱う。
 *************************************************************/
void interrupt isr(void){

	if (RCREG != 0x0d && RCREG != 0x0a){		//CR+LF
		Buffer[t] = RCREG;		//受信データを文字配列に移動
		t++;
		if(t > 40)				//バッファオーバー時の処理
			t = 0;				//文字配列を初期化
	}
	else{
		Buffer[t] = 0;			//文字列終了
		series_flg = 1;			//受信完了のフラグを立てる
		t = 0;					//文字配列を初期化
	}
		RCIF = 0;				//割り込みフラグをクリア
}
